@@ -39,7 +39,7 @@ __all__ = [
 ]
 
 class Arg(object):
-    
+
     def __init__(self, name, optional=False, default=None, validator=None):
         # Are these class members unnecessary when using the below func?
         self.name = name
@@ -63,16 +63,14 @@ class Arg(object):
                 data = inargs[self.name]
             # Default assumes optional=True
             if not data:
-                if self.default:
+                if self.optional:
                     return self.default
-                elif self.optional:
-                    return None
                 elif exists:
                     msg = " is required, exists, but is empty"
                 else:
                     msg = " is required but does not exist"
                 raise nudge.publisher.HTTPException(
-                    400, 
+                    400,
                     self.name + msg
                 )
             # Query string args will come in list format, take the first.
@@ -83,14 +81,14 @@ class Arg(object):
                 return self.validator(data)
             except (validate.ValidationError), e:
                 msg = "invalid value for argument '%s': '%s'" % (
-                    self.name, 
+                    self.name,
                     data
                 )
                 if e.message:
                     msg += ': %s' % e.message
                     raise nudge.publisher.HTTPException(400, msg)
         self.argspec = func
-    
+
 class CustomArg(Arg):
 
     def __init__(self, name=None):
@@ -98,11 +96,11 @@ class CustomArg(Arg):
             name = dehump(self.__name__)
         self.name = name
         pass
-    
+
     def argspec(self):
         pass
 
-class String(Arg): 
+class String(Arg):
     """ Standard unicode string. No size restrictions """
 
     def __init__(self, name, optional=False, max_len=None):
@@ -110,30 +108,30 @@ class String(Arg):
         if optional:
             validator = validate.String()
         super(String, self).__init__(
-            name, 
-            optional=optional, 
+            name,
+            optional=optional,
             validator=validator
         )
 
-class Dict(Arg): 
+class Dict(Arg):
 
-    def __init__(self, name, min_=None, max_=None, 
+    def __init__(self, name, min_=None, max_=None,
                  optional=False):
         super(Dict, self).__init__(
-            name, 
-            optional, 
-            default=None, 
+            name,
+            optional,
+            default=None,
             validator=validate.Dict(min_, max_)
         )
 
-class List(Arg): 
+class List(Arg):
 
-    def __init__(self, name, min_=None, max_=None, 
+    def __init__(self, name, min_=None, max_=None,
                  optional=False):
         super(List, self).__init__(
-            name, 
-            optional, 
-            default=None, 
+            name,
+            optional,
+            default=None,
             validator=validate.List(min_, max_)
         )
 
@@ -141,9 +139,9 @@ class Boolean(Arg):
 
     def __init__(self, name, default=None, optional=False):
         super(Boolean, self).__init__(
-            name, 
-            optional, 
-            default, 
+            name,
+            optional,
+            default,
             validate.Boolean()
         )
 
@@ -151,9 +149,9 @@ class Date(Arg):
 
     def __init__(self, name, default=None, optional=False):
         super(Date, self).__init__(
-            name, 
-            optional, 
-            default, 
+            name,
+            optional,
+            default,
             validate.Date()
         )
 
@@ -161,25 +159,25 @@ class Json(Arg):
 
     def __init__(self, name, default=None, optional=False):
         super(Json, self).__init__(
-            name, 
-            optional, 
-            default, 
+            name,
+            optional,
+            default,
             validate.Json()
         )
 
-class Integer(Arg): 
+class Integer(Arg):
 
-    def __init__(self, name, min_=None, max_=None, 
+    def __init__(self, name, min_=None, max_=None,
                  default=None, optional=False):
         super(Integer, self).__init__(
-            name, 
-            optional, 
-            default, 
+            name,
+            optional,
+            default,
             validate.Int(min_, max_)
         )
 
 class ClientIp(CustomArg):
-    
+
     def __init__(self):
         def func(req, inargs):
             client_ip = req.headers.get("X-Forwarded-For", "") or req.remote_ip
@@ -196,6 +194,20 @@ class RequestHeader(CustomArg):
             return req.headers.get(header)
         self.argspec = func
 
+class Cookie(CustomArg):
+
+    def __init__(self, cookie):
+        def func(req, inargs):
+            '''
+            Right now just return the first item of the list
+            (no support for multivalued cookies)
+            '''
+            cook = req.cookies.get(cookie)
+            if cook and len(cook) >= 1:
+                return cook[0]
+            # Else we return None :)
+        self.argspec = func
+
 class Action(Arg):
     actions = [
         "add",
@@ -220,7 +232,7 @@ class JsonBody(CustomArg):
     """ Checks that there's a valid JSON message body, converts this into
         a dict. It optionally adds specified entries to the dict and
         ultimately returns it.
-       
+
         Note: if the request is a POST or a PUT, and the content type is
         application/json, the json body will be decoded and added to the arg
         dict (so you dont need to use this, you can use normal args).
@@ -231,12 +243,12 @@ class JsonBody(CustomArg):
 
     def __init__(self, optional=False, extend={}):
         def func(req, inargs):
-            if not req.body: 
+            if not req.body:
                 if optional:
                     return None
                 else:
                     raise nudge.publisher.HTTPException(
-                        400, 
+                        400,
                         "json body is not optional"
                     )
             json_body = _get_json_body(req)
@@ -246,8 +258,8 @@ class JsonBody(CustomArg):
         self.argspec = func
 
 class JsonBodyField(CustomArg):
-    """ DEPRECATED - 
-        
+    """ DEPRECATED -
+
         Note: if the request is a POST or a PUT, and the content type is
         application/json, the json body will be decoded and added to the arg
         dict (so you dont need to use this, you can use normal args).
@@ -263,7 +275,7 @@ class JsonBodyField(CustomArg):
             except (KeyError, TypeError):
                 if not optional:
                     raise nudge.publisher.HTTPException(
-                        400, 
+                        400,
                         "json body does not contain '%s'" % fieldname
                     )
             return None
@@ -275,10 +287,7 @@ def _get_json_body(req):
     if '_body_dict' in req.arguments:
         return req.arguments['_body_dict']
 
-    if not req.body or \
-        not req.headers.get(
-            "Content-Type", 
-            '').startswith("application/json"):
+    if not req.body:
         return None
 
     try:
