@@ -20,17 +20,18 @@ import cgi
 import logging
 import re
 import sys
-import types
 import time
 import types
 import urllib
 import warnings
+try:
+    import cStringIO as StringIO
+except ImportError:
+    import StringIO
 
 import nudge.json
 import nudge.log
-import nudge.arg as args
-from nudge.renderer import Json
-from nudge.validator import ValidationError
+from nudge.renderer import Json, RequestAwareRenderer
 from nudge.json import Dictomatic
 from nudge.error import handle_exception, HTTPException, JsonErrorHandler,\
     DEFAULT_ERROR_CODE, DEFAULT_ERROR_CONTENT_TYPE, DEFAULT_ERROR_CONTENT, responses
@@ -60,13 +61,16 @@ class Endpoint(object):
     sequential = []
     named = {}
 
-    def __init__(self, name=None, method=None, uri=None, uris=None, function=None, 
-                 args=None, exceptions=None, renderer=None):
+    def __init__(self, name=None, method=None, uri=None, uris=None,
+                 function=None, args=None, exceptions=None, renderer=None):
         # Someday support unicode here, for now only bytestrings.
         assert isinstance(name, str)
         assert isinstance(method, str)
-        assert (not uris and isinstance(uri, str)) or (not uri and isinstance(uris, types.ListType)), "Endpoints must have either a uri or uris, but not both"
-        assert callable(function) or isinstance(function, str), "function must be callable or a string, but was %s" % type(function)
+        assert (not uris and isinstance(uri, str)) or \
+            (not uri and isinstance(uris, types.ListType)), \
+            "Endpoints must have either a uri or uris, but not both"
+        assert callable(function) or isinstance(function, str), \
+            "function must be callable or a string, but was %s" % type(function)
 
         assert not exceptions or isinstance(exceptions, dict), \
             "exceptions must be a dict, but was type %s" % type(exceptions)
@@ -121,17 +125,25 @@ class WSGIHeaders(dict):
         self.update(*args, **kwargs)
 
     def __getitem__(self, key):
-        return super(WSGIHeaders, self).__getitem__(WSGIHeaders.normalize_name(key))
+        return super(WSGIHeaders, self).__getitem__(
+            WSGIHeaders.normalize_name(key)
+        )
 
     def __setitem__(self, key, value):
-        return super(WSGIHeaders, self).__setitem__(WSGIHeaders.normalize_name(key), value)
+        return super(WSGIHeaders, self).__setitem__(
+            WSGIHeaders.normalize_name(key), value
+        )
 
     def get(self, key, default=None):
-        return super(WSGIHeaders, self).get(WSGIHeaders.normalize_name(key), default)
+        return super(WSGIHeaders, self).get(
+            WSGIHeaders.normalize_name(key), default
+        )
 
     def set(self, key, value):
-        return super(WSGIHeaders, self).set(WSGIHeaders.normalize_name(key), value)
-    
+        return super(WSGIHeaders, self).set(
+            WSGIHeaders.normalize_name(key), value
+        )
+
     @staticmethod
     def normalize_name(n):
         return n.lower().replace('-','_')
@@ -140,13 +152,14 @@ class WSGIHeaders(dict):
 class WSGIRequest(object):
 
     def __init__(self, req_dict):
+        # Should we use dictomatic here? (tiny slowdown)
         self.req = Dictomatic.wrap(req_dict)
         self.start_time = time.time()
         self.method = self.req['REQUEST_METHOD']
         self.remote_ip = self.req['REMOTE_ADDR']
         self.body = self.req['wsgi.input'].read()
         self._buffer = ''
-    
+
     @lazyprop
     def path(self):
         return self.req['PATH_INFO']
@@ -154,8 +167,8 @@ class WSGIRequest(object):
     @lazyprop
     def uri(self):
         return '%s://%s%s' % (
-                self.req['wsgi.url_scheme'], 
-                self.req['HTTP_HOST'], 
+                self.req['wsgi.url_scheme'],
+                self.req['HTTP_HOST'],
                 self.req['PATH_INFO']
             )
 
@@ -169,31 +182,80 @@ class WSGIRequest(object):
                 _headers['Content-Type'] = v
         return _headers
 
+    @lazyprop
+    def cookies(self):
+        ''' I am assuming it is valid to have multiple cookies with the same
+        name even though that seems silly.
+
+        Also the individual cookies are split on the FIRST = so as to support
+        later = for unknown benefit.
+
+        TODO - support/test with unicode?
+             - Maybe nuke the HTTP_COOKIE from regular headers?
+        '''
+        _cookies = {}
+        if 'HTTP_COOKIE' in self.req:
+            for cookie in self.req['HTTP_COOKIE'].split(';'):
+                cookie = cookie.split('=')
+                if len(cookie) < 2: # Empty values will still be >= 2
+                    continue
+                if cookie[0] in _cookies:
+                    _cookies[cookie[0].strip()].append(
+                        '='.join(cookie[1:]).strip()
+                    )
+                else:
+                    _cookies[cookie[0].strip()] = ['='.join(cookie[1:]).strip()]
+        return _cookies
+
     @lazyprop
     def arguments(self):
         _arguments = {}
         try:
-            tmp = self.req.get('QUERY_STRING', '').split('&')
-            tmp = [a.split('=') for a in tmp]
-            tmp = filter(lambda x: len(x) == 2, tmp)
-            for a in tmp:
-                if a[0] in _arguments:
-                    _arguments[a[0]].append(a[1])
-                else:
-                    _arguments[a[0]] = [a[1]]
+            # First url decode
+            tmp = self.req.get('QUERY_STRING', '')
+            if tmp:
+                # First make unicode.
+                # Consider making the unicode decoding type a Nudge option.
+                tmp = unicode(tmp, encoding="utf-8")
+                # make sure to split before unquoting
+                # to handle arg keys/values that contain & or =
+                tmp = tmp.split('&')
+                tmp = [a.split('=') for a in tmp]
+                tmp = [map(urllib.unquote_plus, a) for a in tmp]
+                # Only keep args with k and v. f= will stay and be [u'f', u'']
+                tmp = filter(lambda x: len(x) == 2, tmp)
+                for k, v in tmp:
+                    if k in _arguments:
+                        _arguments[k].append(v)
+                    else:
+                        _arguments[k] = [v]
         except (Exception), e:
-            _log.error(
-                'problem making arguments out of QUERY_STRING: %s', 
+            _log.exception(
+                "problem making arguments out of QUERY_STRING: %s",
                 self.req['QUERY_STRING']
             )
 
-        # add any arguments from JSON body
         if self.method in ('POST', 'PUT') and self.body:
             content_type = self.headers.get("Content-Type", '')
-            # add form args
+            # TODO make sure these come out as unicode
             if content_type.startswith("application/x-www-form-urlencoded"):
                 for name, values in cgi.parse_qs(self.body).iteritems():
                     _arguments.setdefault(name, []).extend(values)
+            # multipart form
+            elif content_type.startswith("multipart/form-data"):
+                try:
+                    fs = cgi.FieldStorage(
+                        fp=StringIO.StringIO(self.body),
+                        environ=self.req,
+                        keep_blank_values=1
+                    )
+                    for k in fs.keys():
+                        _arguments[k] = fs[k].value
+                except:
+                    _log.exception(
+                        "problem parsing multipart/form-data"
+                    )
+            # add any arguments from JSON body
             elif content_type.startswith("application/json"):
                 try:
                     body = nudge.json.json_decode(self.body)
@@ -202,7 +264,6 @@ class WSGIRequest(object):
 
                 except (ValueError):
                     raise HTTPException(400, "body is not JSON")
-
         return _arguments
 
     def write(self, content):
@@ -233,8 +294,12 @@ def redirect(uri, headers=None):
 
 class ServicePublisher(object):
 
-    def __init__(self, fallbackapp=None, endpoints=None, \
-                 debug=False, options=None):
+    def __init__(self, fallbackapp=None, endpoints=None,
+                 debug=False, options=None, default_error_handler=None):
+        # Note fallback app needs to be a wsgi-compatible callable
+        if fallbackapp:
+            assert callable(fallbackapp), "Fallback app must be callable"
+        self._fallbackapp = fallbackapp
         self._debug = debug
         if self._debug:
             _log.setLevel(logging.DEBUG)
@@ -243,18 +308,18 @@ class ServicePublisher(object):
             assert isinstance(endpoints, list), "endpoints must be a list"
             for ep in endpoints:
                 self.add_endpoint(ep)
-        # TODO Fix fallback app here and below
-        self._fallbackapp = fallbackapp
 
+        if not default_error_handler:
+            default_error_handler = JsonErrorHandler
         self._options = Dictomatic({
-            "default_error_handler": JsonErrorHandler(),
+            "default_error_handler": default_error_handler,
         })
 
         if options:
             assert isinstance(options, dict), "options must be of type dict"
             self._options.update(options)
         self.verify_options()
-            
+
     def verify_options(self):
         msg = "Default exception handler "
         assert self._options.default_error_handler, msg + "must exist"
@@ -293,14 +358,18 @@ class ServicePublisher(object):
         args = req.QUERY_STRING.split('=')
 
     def __call__(self, environ, start_response):
-        """ 
+        '''
             This is called by each request to the server.
             This MUST return a valid HTTP response under all circumstances.
-        """
-        if isinstance(environ, types.DictType):
-            req = WSGIRequest(environ)
-        else:
-            req = environ
+
+            We expect environ to be a valid wgsi python dictionary.
+        '''
+        req = WSGIRequest(environ)
+
+#        if isinstance(environ, types.DictType):
+#            req = WSGIRequest(environ)
+#        else:
+#            req = environ
 
         # main exception handler to ensure client gets valid response.
         # defer any mutation of the request object (incl. writes to the client)
@@ -310,7 +379,7 @@ class ServicePublisher(object):
         final_content = ""
         endpoint = None
         try:
-            # allow '_method' query arg to overide method
+            # allow '_method' query arg to override method
             method = req.method
             if '_method' in req.arguments:
                 method = req.arguments['_method'][0].upper()
@@ -325,21 +394,20 @@ class ServicePublisher(object):
                     break
 
             if not match:
-                # TODO: Handle HTTPException in new world exceptions
-                raise HTTPException(404)
-                #
-                # Fallback app is untested with WSGI/EVENTLET
-                # FIXTHIS!!
-                #
-                # if self._fallbackapp:
-                    # _log.debug("falling through: %s %s" % (method, req.uri))
-                    # return self._fallbackapp(event_req, start_response)
-                # else:
-                    # raise HTTPException(404)
+                if self._fallbackapp:
+                    _log.debug("Using fallback app for request: (%s) (%s)" % \
+                               (method, req.uri))
+                    # Since recreating a stringio from the request body can be
+                    # expensive, we only want to do this if we fallback.
+                    environ['wsgi.input'] = StringIO.StringIO(req.body)
+                    return self._fallbackapp(environ, start_response)
+                else:
+                    raise HTTPException(404)
 
             # convert all values in req.arguments from lists to scalars,
             # then combine with path args.
-            arguments = dict((k, v[0]) for k, v in req.arguments.iteritems() if isinstance(v, list))
+            arguments = dict((k, v[0]) for k, v in req.arguments.iteritems()\
+                if isinstance(v, list) and len(v) > 0)
             inargs = dict(match.groupdict(), **arguments)
 
             # compile positional arguments
@@ -353,51 +421,65 @@ class ServicePublisher(object):
                 r = arg.argspec(req, inargs)
                 if r != None:
                     kwargs[argname] = r
-                        
+
             # invoke the service endpoint
             result = endpoint(*args, **kwargs)
 
             # TODO make sure this works with unicode
             _log.debug(_gen_trace_str(endpoint.function, args, kwargs, result))
-            
-            r = endpoint.renderer(result)
+
+            if isinstance(endpoint.renderer, RequestAwareRenderer):
+                r = endpoint.renderer(req, result)
+            else:
+                r = endpoint.renderer(result)
             content, content_type, code, extra_headers = \
                 r.content, r.content_type, r.http_status, r.headers
 
         except (Exception), e:
             error_response = None
+            logged_trace = False
             #
             # Try to use this endpoint's exception handler(s)
             # If the raised exception is not mapped in this endpoint, or
-            # this endpoint raises an exception when trying to handle, 
+            # this endpoint raises an exception when trying to handle,
             # we will then try to the default handler, and ultimately
             # fallback to the self._options.default_error_response, which
             # is guaranteed to be valid at app initialization.
             #
             if endpoint and endpoint.exceptions:
                 try:
-                    error_response = handle_exception(e, endpoint.exceptions)
+                    error_response = handle_exception(
+                        e, endpoint.exceptions,
+                        default_handler=self._options.default_error_handler
+                    )
+                    if not error_response:
+                        raise
                 except (Exception), e:
-                    # TODO this may log too loudly
-                    _log.exception("Endpoint failed to handle exception")
+                    _log.exception(
+                        "Endpoint (%s) failed to handle exception" % endpoint.name
+                    )
+                    logged_trace = True
 
             if not error_response:
                 try:
                     # Try one more time to handle a base exception
-                    error_response = self._options.default_error_handler(e)
+                    handler = self._options.default_error_handler()
+                    error_response = handler(e)
                 except (Exception), e:
-                    _log.exception(
+                    _log.error(
                         "Default error handler failed to handle exception")
+                    if logged_trace is False:
+                        _log.exception(e)
 
             code, content_type, content, extra_headers = \
                 error_response or self._options.default_error_response
 
         final_content = _finish_request(
-            req, 
-            start_response, 
-            code, 
-            content_type, 
-            content, 
+            req,
+            start_response,
+            code,
+            content_type,
+            content,
             extra_headers
         )
 
@@ -425,7 +507,7 @@ def _finish_request(req, start_response, code, content_type, content, headers):
         code = DEFAULT_ERROR_CODE
 
     start_response(
-        str(code) + ' ' + responses[code], 
+        str(code) + ' ' + responses[code],
         final_headers
     )
     return content
@@ -464,21 +546,21 @@ def serve(service_description, args=None):
     parser.add_option(
         "-p", "--port", dest="port", help="port to run on", default=8080)
     parser.add_option(
-        "-s", "--server", dest="server", 
+        "-s", "--server", dest="server",
         help="which http server to use", default="eventlet")
     # parser.add_option(
-        # "-t", "--threaded", dest="threaded", 
+        # "-t", "--threaded", dest="threaded",
         # help="run as a threaded server (paste)", default=False)
     parser.add_option(
-        "-n", "--threads", dest="threads", 
-        help="number of worker threads (only honored if --threaded)", 
+        "-n", "--threads", dest="threads",
+        help="number of worker threads (only honored if --threaded)",
         default=15)
     parser.add_option(
-        "-b", "--backlog", dest="backlog", 
-        help="maximum number of queued connections (only used if --threaded)", 
+        "-b", "--backlog", dest="backlog",
+        help="maximum number of queued connections (only used if --threaded)",
         default=5)
     parser.add_option(
-        "-d", "--debug", action="store_true", dest="debug", 
+        "-d", "--debug", action="store_true", dest="debug",
         help="setup nudge in debug mode (extra color logging)", default=False)
 
     (options, args) = parser.parse_args(args)
@@ -499,7 +581,7 @@ def serve(service_description, args=None):
         threads = int(options.threads)
         backlog = int(options.backlog)
         paste.httpserver.serve(
-            sp, host=None, port=port, use_threadpool=True, 
+            sp, host=None, port=port, use_threadpool=True,
             threadpool_workers=threads, threadpool_options=None,
             request_queue_size=backlog
         )
