@@ -15,16 +15,17 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
-
 import base64
+import types
 from cStringIO import StringIO
 from nudge.json import json_encode
-from nudge.error import HTTPException
+from nudge.error import HTTPException, SecurityException
 
 __all__ = [
-    'ExceptionRenderer',
     'Result', 
+    'RequestAwareRenderer', 
     'Json', 
+    'InsecureJson', 
     'Redirect',
     'CSS',
     'Img',
@@ -46,10 +47,37 @@ class Result(object):
             headers = {}
         self.headers = headers
 
-class Json(object):
-    """ Default Nudge HTTP Content Type. Encodes the entire endpoint
-        result as json, and returns """
+class RequestAwareRenderer(object):
 
+    def __call__(self, req, result):
+        pass
+
+
+class Json(object):
+    ''' Default Nudge HTTP Content Type. Encodes the entire endpoint
+        result as json, and returns. 
+        
+        Note this renderer will not allow you to pass back a top level
+        list or tuple (javascript array) because it is insecure, and could
+        allow a CSRF attack. See here:
+        http://flask.pocoo.org/docs/security/#json-security    
+    '''
+    def __call__(self, result):
+        if result == None:
+            raise HTTPException(404)
+        if isinstance(result, (types.ListType, types.TupleType)):
+            raise SecurityException(
+                'Results that encode as json arrays are not '+\
+                'allowed for security concerns'
+            )
+        return Result(
+            content=json_encode(result),
+            content_type='application/json; charset=UTF-8',
+            http_status=200,
+        )
+
+class InsecureJson(object):
+    ''' See the notes above for Json. This does not check returned types. '''
     def __call__(self, result):
         if result == None:
             raise HTTPException(404)
